/*
 * jGuild Project: jRPM
 * Released under the Apache License ( http://www.apache.org/LICENSE )
 */
package com.jguild.jrpm.io;

import java.io.DataInputStream;
import java.io.IOException;

import org.apache.log4j.Logger;

import com.jguild.jrpm.io.constant.EnumIf;
import com.jguild.jrpm.io.constant.RPMSignatureTag;

/**
 * RPM Signature.
 * 
 * @version $Id: RPMSignature.java,v 1.6 2004/05/06 20:59:23 mkuss Exp $
 */
public class RPMSignature extends Header {

    private static final Logger logger = Logger.getLogger(RPMSignature.class);

    /**
     * Creates a new RPMSignature object from an input stream
     * 
     * @param inputStream
     *           The input stream
     * 
     * @throws IOException
     *            if an error occurs on reading informations out of the stream
     */
    public RPMSignature(DataInputStream inputStream) throws IOException {
        super(inputStream);

        // Make signature size modulo 8 = 0
        long fill = (size % 8L);

        if (fill != 0) {
            fill = 8 - fill;
        }

        if (logger.isDebugEnabled())
                logger.debug("skip " + fill + " bytes for signature");

        inputStream.skip(fill);
        size += fill;
    }

    /*
     * @see com.jguild.jrpm.io.Header#getKnownTagNames()
     */
    public static String[] getKnownTagNames() {
        return RPMSignatureTag.getEnumNames();
    }

    /*
     * @see com.jguild.jrpm.io.Header#getTagIdForName(java.lang.String)
     */
    public long getTagIdForName(String tagname) {
        EnumIf enum = RPMSignatureTag.getEnumByName(tagname);

        if (enum == null) { throw new IllegalArgumentException(
                "unknown tag with name <" + tagname + ">"); }

        return enum.getId();
    }

    /*
     * @see com.jguild.jrpm.io.Header#getTagNameForId(long)
     */
    public String getTagNameForId(long tagid) {
        EnumIf enum = RPMSignatureTag.getEnumById(tagid);

        if (enum == null) { throw new IllegalArgumentException(
                "unknown tag with id <" + tagid + ">"); }

        return enum.getName();
    }
    /* 
     * @see com.jguild.jrpm.io.Header#isValidTag(long)
     */
    public boolean isValidTag(long tagid) {
        EnumIf enum = RPMSignatureTag.getEnumById(tagid);
        if (enum == null) {
            return false;
        }
        return true;
    }
    
    /* 
     * @see com.jguild.jrpm.io.Header#isValidTag(java.lang.String)
     */
    public boolean isValidTag(String tagname) {
        EnumIf enum = RPMSignatureTag.getEnumByName(tagname);
        if (enum == null) {
            return false;
        }
        return true;
    }
}
